package me.savvy.rixa.modules.music;

import com.sedmelluq.discord.lavaplayer.player.AudioPlayer;
import com.sedmelluq.discord.lavaplayer.player.event.AudioEventAdapter;
import com.sedmelluq.discord.lavaplayer.track.AudioTrack;
import com.sedmelluq.discord.lavaplayer.track.AudioTrackEndReason;
import lombok.Getter;
import lombok.Setter;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

/**
 * Created by Timber on 6/13/2017.
 */
public class TrackScheduler extends AudioEventAdapter {
        @Getter @Setter
        private boolean repeating = false;
        final AudioPlayer player;
        public final Queue<AudioTrack> queue;
        public AudioTrack lastTrack;

        /**
         * @param player The audio player this scheduler uses
         */
        public TrackScheduler(AudioPlayer player) {
            this.player = player;
            this.queue = new LinkedList<>();
        }

        /**
         * Add the next track to queue or play right away if nothing is in the queue.
         *
         * @param track The track to play or add to queue.
         */
        public void queue(AudioTrack track)
        {

            // Calling startTrack with the noInterrupt set to true will start the track only if nothing is currently playing. If
            // something is playing, it returns false and does nothing. In that case the player was already playing so this
            // track goes to the queue instead.
            if (!player.startTrack(track, true))
            {
                queue.offer(track);
            }
        }

        /**
         * Start the next track, stopping the current one if it is playing.
         */
        public AudioTrack nextTrack()
        {
            // Start the next track, regardless of if something is already playing or not. In case queue was empty, we are
            // giving null to startTrack, which is a valid argument and will simply stop the player.
            AudioTrack track = queue.poll();
            player.startTrack(track, false);
            return track;
        }

        @Override
        public void onTrackEnd(AudioPlayer player, AudioTrack track, AudioTrackEndReason endReason) {
            this.lastTrack = track;
            // Only start the next track if the end reason is suitable for it (FINISHED or LOAD_FAILED)
            if (endReason.mayStartNext) {
                if (repeating)
                    player.startTrack(lastTrack.makeClone(), false);
                else
                    nextTrack();
            }

        }
    
        public void shuffle() {
            Collections.shuffle((List<?>) queue);
        }
    }